﻿using Foundation;
using RestSharp;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using AgClockingClockout.iOS.Helpers;
using UIKit;
using ZXing.Mobile;
using Newtonsoft.Json;
using Acr.UserDialogs;
using static AgClockingClockout.iOS.Helpers.Models;
using Plugin.Permissions.Abstractions;
using Plugin.Permissions;

namespace AgClockingClockout.iOS
{
    public partial class ViewController : UIViewController
    {
        private string ScannedCode { get; set; }
        private bool isOk { get; set; }

        DataHelperModel model = new DataHelperModel();

        public ViewController (IntPtr handle) : base (handle)
        {

        }
        public override void ViewDidLoad ()
        {
            base.ViewDidLoad ();
            // Perform any additional setup after loading the view, typically from a nib.

        }

        public override void DidReceiveMemoryWarning ()
        {
            base.DidReceiveMemoryWarning ();
            // Release any cached data, images, etc that aren't in use.
        }
        private async void ForcePermissions()
        {
            var status = await CrossPermissions.Current.CheckPermissionStatusAsync(Permission.Location);


            switch (status)
            {
                case PermissionStatus.Granted:

                    ScanCode();
                    //HandleScanResult("2873");
                    break;

                case PermissionStatus.Denied:

                    this.InvokeOnMainThread(() =>
                    {
                        UserDialogs.Instance.Alert("Location is required. Current status: Denied", "Warning", "OK");
                    });

                    await CrossPermissions.Current.RequestPermissionsAsync(Permission.Location);
                    break;

                case PermissionStatus.Disabled:

                    this.InvokeOnMainThread(() =>
                    {
                        UserDialogs.Instance.Alert("Location is required. Current status: Disabled", "Warning", "OK");
                    });

                    await CrossPermissions.Current.RequestPermissionsAsync(Permission.Location);
                    break;

                case PermissionStatus.Restricted:

                    this.InvokeOnMainThread(() =>
                    {
                        UserDialogs.Instance.Alert("Location is required. Current status: Restricted", "Warning", "OK");
                    });

                    await CrossPermissions.Current.RequestPermissionsAsync(Permission.Location);
                    break;

                case PermissionStatus.Unknown:

                    this.InvokeOnMainThread(() =>
                    {
                        UserDialogs.Instance.Alert("Location is required. Current status: Unknown", "Warning", "OK");
                    });

                    await CrossPermissions.Current.RequestPermissionsAsync(Permission.Location);
                    break;
            }




            //if (status == PermissionStatus.Granted)
            //{
               
            //}
            //else
            //{
            //    if (await CrossPermissions.Current.ShouldShowRequestPermissionRationaleAsync(Permission.Location))
            //    {
            //        this.InvokeOnMainThread(() =>
            //        {
            //            UserDialogs.Instance.Alert("Location is required", "Warning", "OK");
            //        });
            //    }

            //    var results = await CrossPermissions.Current.RequestPermissionsAsync(Permission.Location);
            //    status = results[Permission.Location];
            //}
        }
        void ScanCode()
        {
            MobileBarcodeScanner scanner = new MobileBarcodeScanner(this);

            scanner.UseCustomOverlay = false;
            scanner.TopText = "Hold camera up to QR code to scan";
            scanner.BottomText = "QR code will automatically scan";

            Task.Run(async () =>
            {
                var result = await scanner.Scan();

                if (string.IsNullOrEmpty(result.Text))
                {
                    UserDialogs.Instance.Alert("Operation cancelled or code is unreadable", "Warning");
                    return;
                }

                this.InvokeOnMainThread(() =>
                {
                    UserDialogs.Instance.ShowLoading("Please wait");
                });

                ScannedCode = result.Text;

                  HandleScanResult(result.Text);

            }).ContinueWith(result =>
                this.InvokeOnMainThread(() =>
                {
                    UserDialogs.Instance.HideLoading();

                    if (string.IsNullOrEmpty(ScannedCode))
                    {
                        UserDialogs.Instance.Alert("Operation cancelled or code is unreadable", "Warning");
                        return;
                    }

                    if (isOk)
                    {
                        PartTwoController controller =
                            this.Storyboard.InstantiateViewController
                                ("PartTwoController") as PartTwoController;

                        model.AccountNumber = ScannedCode;

                        controller.DataHelper = JsonConvert.SerializeObject(model);

                       // controller.DataHelper = "some text from qr controller";

                        this.NavigationController.SetNavigationBarHidden(true, true);

                        this.NavigationController.PushViewController(controller, true);
                    }

                }));

        }
        async void  HandleScanResult(string result)
        {
            string msg = "";

            isOk = false;
            if (!string.IsNullOrEmpty(result))
            {

                var position = await GpsHelper.GetCurrentPosition();

                if(position == null)
                {
                    position.Longitude = 0;
                    position.Latitude = 0;

                    //Comment out below for the system to work without location tracking funcion in place

                    this.InvokeOnMainThread(() =>
                    {
                        var av = new UIAlertView("Error", "Please allow location services.", null, "OK", null);
                        av.Show();
                        return;
                    });
                }

                if (result.Contains("http://"))
                {
                    result = result.Replace("http://", "");
                }

                API client = new API();

                var request = new RestRequest("api/ams/codes", Method.GET);
                request.AddParameter("accountNumber", result);
                request.AddParameter("longitude", position.Longitude);
                request.AddParameter("latitude", position.Latitude);

                IRestResponse response = client.GetClient().Execute(request);

                if (response.IsSuccessful)
                {
                    model.Codes = new List<DepartmentCode>();
                    model.Codes = JsonConvert.DeserializeObject<List<Models.DepartmentCode>>(response.Content);

                    if (model.Codes.Count > 0)
                    {
                        isOk = true;
                    }
                    else
                    {
                        msg = "No codes available for scanned site.";

                        this.InvokeOnMainThread(() =>
                        {
                            var av = new UIAlertView("Warning", msg, null, "OK", null);
                            av.Show();
                        });

                        this.InvokeOnMainThread(() =>
                        {
                            UserDialogs.Instance.HideLoading();
                        });
                    }
                }
                else
                {
                    this.InvokeOnMainThread(() =>
                    {
                        UserDialogs.Instance.HideLoading();

                        var av = new UIAlertView("Error", response.Content, null, "OK", null);
                        av.Show();
                    });
                }
            }
            else
            {
                msg = "Scanning Canceled!";

                this.InvokeOnMainThread(() =>
                {
                    var av = new UIAlertView("Barcode Result", msg, null, "OK", null);
                    av.Show();
                });
            }

        }

        partial void BtnStartScan_TouchUpInside(UIButton sender)
        {
            ForcePermissions();
        }

        //partial void (UIButton sender)
        //{
        //  
        //}
    }
}